@@ -103,6 +103,8 @@ enum class EventType : uint32_t {
      */
     InputMethodGroupChanged = InstanceEventFlag | 0x1,
     InputMethodGroupAboutToChange = InstanceEventFlag | 0x2,
+    UIChanged = InstanceEventFlag | 0x3,
+    CheckUpdate = InstanceEventFlag | 0x4,
 };
 
 /**
@@ -168,9 +170,7 @@ protected:
 class FCITXCORE_EXPORT KeyEventBase : public InputContextEvent {
 public:
     KeyEventBase(EventType type, InputContext *context, Key rawKey,
-                 bool isRelease = false, int time = 0)
-        : InputContextEvent(context, type), key_(rawKey.normalize()),
-          origKey_(key_), rawKey_(rawKey), isRelease_(isRelease), time_(time) {}
+                 bool isRelease = false, int time = 0);
     KeyEventBase(const KeyEventBase &) = default;
 
     /**
@@ -180,20 +180,63 @@ public:
      */
     Key key() const { return key_; }
 
+    /**
+     * It will automatically be called if input method layout does not match the
+     * system keyboard layout.
+     *
+     * @param key p_key:...
+     */
     void setKey(const Key &key) {
         key_ = key;
         forward_ = true;
     }
 
+    /**
+     * It is designed for faking the key event. Normally should not be used.
+     *
+     * @param key key event to override.
+     * @since 5.0.4
+     */
+    void setRawKey(const Key &key) {
+        rawKey_ = key;
+        key_ = key.normalize();
+        forward_ = true;
+    }
+
+    /**
+     * It is designed for overriding the key forward option. Normally should not
+     * be used.
+     *
+     * @param forward
+     * @since 5.0.4
+     */
+    void setForward(bool forward) { forward_ = forward; }
+
     /**
      * Key event regardless of keyboard layout conversion.
      *
      * @return fcitx::Key
      */
     Key origKey() const { return origKey_; }
+
+    /**
+     * Key event after layout conversion.
+     *
+     * Basically it is the "unnormalized" key event.
+     *
+     * @return fcitx::Key
+     */
     Key rawKey() const { return rawKey_; }
     bool isRelease() const { return isRelease_; }
     int time() const { return time_; }
+
+    /**
+     * If true, the key that produce character will commit a string.
+     *
+     * This is currently used by internal keyboard layout translation.
+     *
+     * @return bool
+     */
     bool forward() const { return forward_; }
 
 protected:
@@ -211,7 +254,7 @@ public:
                        isRelease, time) {}
 
     void filter() { filtered_ = true; }
-    virtual bool filtered() const { return filtered_; }
+    bool filtered() const override { return filtered_; }
     void filterAndAccept() {
         filter();
         accept();
@@ -235,7 +278,7 @@ public:
         : InputContextEvent(context, EventType::InputContextCommitString),
           text_(text) {}
 
-    const std::string text() const { return text_; }
+    const std::string &text() const { return text_; }
 
 protected:
     std::string text_;
@@ -340,6 +383,27 @@ public:
         : Event(EventType::InputMethodGroupAboutToChange) {}
 };
 
+class FCITXCORE_EXPORT UIChangedEvent : public Event {
+public:
+    UIChangedEvent() : Event(EventType::UIChanged) {}
+};
+
+class FCITXCORE_EXPORT CheckUpdateEvent : public Event {
+public:
+    CheckUpdateEvent() : Event(EventType::CheckUpdate) {}
+
+    /// Make checking update short circuit. If anything need a refresh, just
+    /// simply break.
+    void setHasUpdate() {
+        filtered_ = true;
+        accept();
+    }
+    bool filtered() const override { return filtered_; }
+
+private:
+    bool filtered_ = false;
+};
+
 class FCITXCORE_EXPORT CapabilityEvent : public InputContextEvent {
 public:
     CapabilityEvent(InputContext *ic, EventType type, CapabilityFlags oldFlags,
